/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.security;

import org.jboss.logging.Logger;
import org.jboss.security.AuthenticationManager;
import org.jboss.security.SecurityAssociation;

import java.security.Principal;
import javax.security.auth.Subject;

/**
 * The AuthenticationInterceptor authenticates the caller.
 *
 * @author <a href="bill@jboss.org">Bill Burke</a>
 * @author <a href="on@ibis.odessa.ua">Oleg Nitz</a>
 * @author <a href="mailto:Scott.Stark@jboss.org">Scott Stark</a>.
 * @author <a href="mailto:dain@daingroup.com">Dain Sundstrom</a>.
 * @version $Revision: 1.4.2.1.2.1 $
 */
public class AuthenticationInterceptor implements org.jboss.aop.advice.Interceptor
{
   protected Logger log = Logger.getLogger(this.getClass());
   protected AuthenticationManager authenticationManager;

   public AuthenticationInterceptor(AuthenticationManager manager)
   {
      authenticationManager = manager;
   }

   public String getName()
   {
      return "AuthenticationInterceptor";
   }

   /**
    * Authenticates the caller using the principal and credentials in the
    * Infocation if thre is a security manager and an invcocation method.
    */
   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      Principal principal = (Principal) invocation.getMetaData("security", "principal");
      Object credential = invocation.getMetaData("security", "credential");

      if (principal == null)
      {
         principal = SecurityAssociation.getPrincipal();
      }

      if (credential == null)
      {
         credential = SecurityAssociation.getCredential();
      }

      // Check the security info from the method invocation if there is a 
      // security manager and an invocation method.
      Subject subject = new Subject();
      if (authenticationManager != null &&
      !authenticationManager.isValid(principal, credential, subject))
      {
         // Authentication has failed
         String message = "Authentication exception, principal=" + principal;
         log.error(message);
         throw new SecurityException(message);
      }
      else
      {
         // Associate principal and credential with the thread for other beans
         SecurityAssociation.pushSubjectContext(subject, principal, credential);
      }

      Object oldDomain = SecurityContext.currentDomain.get();
      try
      {
         SecurityContext.currentDomain.set(authenticationManager);
         return invocation.invokeNext();
      }
      finally
      {
         SecurityContext.currentDomain.set(oldDomain);
         // so that the principal doesn't keep being associated with thread if the thread is pooled
         SecurityAssociation.popSubjectContext();
      }
   }
}
